module DashboardHelper

  def visit_dashboard
    visit('/')
  end

  def get_sidebar_container(puppet_entity)
    case puppet_entity
      when 'group'
        group_index = 2
      when 'class'
        group_index = 3
      else
        raise "Unknown category #{puppet_entity}"
    end

    session.all('div#sidebar div.group')[group_index]
  end

  def get_puppet_entity(puppet_entity, name)
    group_container = get_sidebar_container(puppet_entity)
    group_container.find('a', :text => name)
  end

  def should_have_puppet_entity(puppet_entity, name)
    group_container = get_sidebar_container(puppet_entity)

    group_container.should have_css('a', :text => /\A#{name}\z/)
  end

  def should_not_have_puppet_entity(puppet_entity, name)
    group_container = get_sidebar_container(puppet_entity)

    group_container.should have_no_css('a', :text => /\A#{name}\z/)
  end

  def should_have_node(node)
    session.find('#header').click_link('Nodes')
    session.should have_css('#main table.main a', :text => /\A#{node}\z/)
  end

  def should_not_have_node(node)
    session.find('#header').click_link('Nodes')
    session.should have_no_css('#main table.main a', :text => /\A#{node}\z/)
  end

  def select_puppet_entity(puppet_entity, name)
    if puppet_entity.eql?('node')
      session.find('#header').click_link('Nodes')
      session.find('#main table.main a', :text => name).click
    else
      get_puppet_entity(puppet_entity, name).click
    end
  end

  def remove_puppet_entity(puppet_entity, name)
    select_puppet_entity(puppet_entity, name)

    session.find('#main').click_link('Delete')

    # give IE time for the alert dialog to appear
    sleep 1
    session.driver.browser.switch_to.alert.accept

    # wait for the page to reload by detecting when the Delete button disappears
    # TODO: remove the :text matcher, as the page reloading can remove the nodes and Capybara enumerates each
    # matched element looking for a :text match
    session.should have_no_css('div#main a', :text => 'Delete')
  end

  def create_class(class_name)
    session.find('#sidebar').click_link('Add classes')
    session.should have_css('h2.header', :text => 'Add classes')
    session.fill_in('custom_node_class_name', :with => "#{class_name}")
    session.find('.typing_filter_add_custom_button_enabled', :visible => true).click()
    session.find('#classes_form').click_button('Add selected classes')
    session.should have_css('.header h2', :text => 'Classes')
  end

  def should_have_class(class_name)
    should_have_puppet_entity('class', class_name)
  end

  def should_not_have_class(class_name)
    should_not_have_puppet_entity('class', class_name)
  end

  def select_class(class_name)
    select_puppet_entity('class', class_name)
  end

  def remove_class(class_name)
    remove_puppet_entity('class', class_name)
  end

  def create_group(group_name)
    session.find('#sidebar').click_link('Add group')
    session.fill_in('node_group_name', :with => "#{group_name}")
    session.find('#param_form').click_button('Create')
    wait_for_modification
  end

  def should_have_group(group_name)
    should_have_puppet_entity('group', group_name)
  end

  def should_not_have_group(group_name)
    should_not_have_puppet_entity('group', group_name)
  end

  def select_group(group_name)
    select_puppet_entity('group', group_name)
  end

  def remove_group(group_name)
    remove_puppet_entity('group', group_name)
  end

  def create_node(node_name)
    session.find('#sidebar').click_link('Add node')
    session.fill_in('node_name', :with => "#{node_name}")
    session.find('#param_form').click_button('Create')
    wait_for_modification
  end

  def select_node(node_name)
    select_puppet_entity('node', node_name)
  end

  def remove_node(node_name)
    remove_puppet_entity('node', node_name)
  end

  def wait_for_modification()
    # waits for the newly-created resource to load - that page includes an Edit button
    wait_for_ajax do
      session.should have_css('div#main a', :text => 'Edit')
    end
  end

  def add_variables(params)
    add_properties('variable', params)
  end

  def add_parameters(params)
    add_properties('parameter', params)
  end

  def add_properties(category, params)
    params_node = session.find('#parameters')

    params.each { |param| params_node.click_link("Add #{category}") }

    i = 0
    params_node.all('tr.parameter', :visible => true).each do |tr|
      tr.find('td.key input').set(params[i][:key])
      tr.find('td.value input').set(params[i][:value])
      i = i + 1

      if i >= params.length
        break
      end
    end
  end

  def remove_variables()
    remove_properties('variable')
  end

  def remove_parameters()
    remove_properties('parameter')
  end

  def remove_properties(category)
    # wait for parameters to appear
    session.find('table#parameters')

    cnt = session.all('form#param_form tr.parameter', :visible => true).length

    for i in cnt.downto(1)
      session.first('form#param_form tr.parameter', :visible => true).click_link("Remove #{category}")

      # wait for JS removal
      j = 0
      while session.all('form#param_form tr.parameter', :visible => true).length >= i && j < 3
        sleep 0.5
        j = j + 1
      end
    end
  end

  def get_group_class(puppet_entity)
  
    group_class = nil
  
    case puppet_entity
      when 'node'
        group_class = 'node'
      when 'group'
        group_class = 'node_group'
      when 'class'
        group_class = 'node_class'
      else
        raise "Unknown puppet_entity #{puppet_entity}"
    end
  
    group_class
  end

  def set_field_with_suggestion(puppet_entity, name)
    form_node = session.find('#param_form')
    input_id = '#' + get_group_class(puppet_entity) + '_autocomplete' 

    # set field
    form_node.first("#{input_id}").set(name)

    suggestions = ".ui-autocomplete .ui-menu-item"

    # wait for suggestions
    session.should have_css(suggestions, :visible => true)

    # select the first suggestion
    session.first(suggestions, :visible => true).click
  end
  
  def remove_field_with_suggestion_values(puppet_entity)
    session.all('#' + puppet_entity + '_membership_table tbody tr a', :text => /\ARemove/, :visible => true).each { |a| a.click() }
  end

  def show_should_have_variables(params)
    variables = session.find('div#main h3', :text => 'Variables')
    variables_table = variables.find(:xpath, './/../table[@class="inspector"]')

    should_have_parameter_or_variables(params, variables_table)
  end

  def show_should_have_parameters(params)
    parameters = session.find('div#main h3', :text => 'Parameters')
    parameters_table = parameters.find(:xpath, './/../table[@class="inspector"]')

    should_have_parameter_or_variables(params, parameters_table)
  end

  def should_have_parameter_or_variables(params, table)
    rows = table.all('tbody tr', :visible => true)

    rows.each_with_index do |tr, i|
      param = tr.all('td tt')

      key = param.at(0).text
      value = param.at(1).text

      key.should == params[i][:key]
      value.should == params[i][:value]
    end

    params.length.should == rows.length
  end

  def should_have_no_parameters
    parameters = session.find('div#main h3', :text => 'Parameters')
    parameters.should have_no_xpath(:xpath, './/../table[@class="inspector"]')
  end

  def select_reports
    session.find("#header").click_link("Reports")
    # wait for reports
    session.should have_css('div#main h2', :text => 'Reports')
  end

  def select_all_reports
    session.find("#all-tab").click_link("All")

    # wait for reports
    session.should have_css('ul#report-tabs li.active a', :text => 'All')
  end

  def check_active_tabs(tab_names)
    links = session.all('ul#report-tabs li a')

    links.each_with_index do |a, i|
      a.text.should == tab_names[i]
    end

    tab_names.length.should == links.length
  end

  def click_active_tabs(tab_names)
    tab_names.each do |tab_name|
      session.find("#report-tabs").click_link(tab_name)

      session.should have_css('ul#report-tabs li.active a', :text => tab_name)
    end
  end

  def select_inventory_search()
    session.find('#header').click_link('Inventory Search')

    session.find('div#main h2', :text => 'Search nodes')
  end

  def get_inspector_table(title)
    classes = session.find('div#main h3', :text => title)
    # the table we're after is adjacent to the header with the specified text
    classes.find(:xpath, './/../table[@class="inspector"]')
  end

  def get_class_param_link(param_name)
    classes_table = get_inspector_table('Classes')
    classes_table.find('a', :text => param_name)
  end

  def click_class_param_link(param_name)
    class_name = get_class_param_link(param_name)
    # look for the last td in the row that contains the link with the param name as its text
    param_td = class_name.all(:xpath, './/../../strong/../../td').last
    param_td.all('a').last.click
  end

  def csrf_test(test_string)
    # log in to the dashboard and establish a session...
    # (note that we need to visit a page that *doesn't* have automatic jQuery
    # CSRF token injection, as that will let the POST through and fail the
    # test)
    visit('/radiator')

    js = "window.test = false;"
    session.evaluate_script(js)

    # ... then initiate a POST *without* the CSRF token ...
    session.evaluate_script(test_string)

    # ... and wait until it fails or succeeds with the desired status.
    wait_until {
      session.evaluate_script('window.test === true')
    }
  end

end
