DEFAULT_ENVIRONMENT = 'production'

module ClassDiscoveryHelper
  def get_available_classes(search_pattern, env = DEFAULT_ENVIRONMENT)

    url = "https://#{SETTINGS.inventory_server}:#{SETTINGS.inventory_port}/" +
          "#{env}/resource_types/#{search_pattern}?kind=class"

    begin
      req = PuppetHttps.get(url, 'pson')
    rescue Net::HTTPServerException => e
      raise "Error performing resource type search: #{e.response.body}"
    rescue Exception
      raise 'Error performing resource type search.'
    end

    JSON.parse(req)
  end
  module_function :get_available_classes
  
  def get_class_details(class_name, env = DEFAULT_ENVIRONMENT)

    url = "https://#{SETTINGS.inventory_server}:#{SETTINGS.inventory_port}/" +
          "#{env}/resource_type/#{class_name}"

    begin
      req = PuppetHttps.get(url, 'pson')
    rescue Net::HTTPServerException => e
      if(e.response.instance_of? Net::HTTPNotFound)
        return nil
      else
        raise "Error performing resource type search: #{e.response.body}"
      end
    rescue Exception
      raise 'Error performing resource type search.'
    end

    JSON.parse(req)
  end
  module_function :get_class_details
  
  class RDoc::Markup::FirstParagraph < RDoc::Markup::Formatter

    def initialize ignore_pattern = nil
      super nil
      
      @ignore_pattern = ignore_pattern
    end
    
    def start_accepting
      @res = []
    end
  
    def end_accepting
      @res.join
    end

    def convert content
      catch :done do
        return super
      end
  
      end_accepting
    end

    def accept_paragraph paragraph
      if @ignore_pattern.nil? || @ignore_pattern.match(paragraph.text).nil? then
        @res << paragraph.text 
        throw :done
      end
    end

    alias accept_indented_paragraph accept_paragraph #consider indented paragraph as paragraph

    alias accept_verbatim accept_paragraph #consider verbatim text as paragraph

    alias accept_blank_line ignore

    alias accept_heading ignore
    
    alias accept_list_start ignore
    
    alias accept_list_end ignore
    
    alias accept_list_item_start ignore
    
    alias accept_list_item_end ignore
    
    alias accept_raw ignore

    alias accept_rule ignore
  end

  def labelize_description(name, description, fallback_label = nil)
    return fallback_label if description.nil?

    markup = RDoc::Markup::FirstParagraph.new /^\s*Class: #{name}\s*$/
    markup.convert(description).sub(/\s\s*/, ' ').sub(/([^\s\.])\.(?:\s.*)?$/, '\1')
  end
  module_function :labelize_description
end
